import java.io.Reader;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.StreamTokenizer;

public class StateMin
{
  static public void main(String [] args)
  {
    // Build a new (initially empty) state table.
    StateTable table = new StateTable();

    // Read a FSM into "table"
    readStateTable(table);

    System.out.println("Initial state table:\n"+table);
    // Build an implication chart from the table
    ImplicationChart chart = new ImplicationChart(table);
    // Run the minimization algorithm
    StateTable minTable = chart.minimizeFSM();
    // Output the resulting state table
    System.out.println("Final state table:\n"+minTable);
  }

  // State table parsing code.  You shouldn't need to read this, if you don't
  // want to.
  static void readStateTable(StateTable t)
  {
    Reader reader = new BufferedReader(new InputStreamReader(System.in));
    StreamTokenizer st = new StreamTokenizer(reader);
    st.eolIsSignificant(false);

    int ttype;
    try {
      ttype = st.nextToken();
      if (ttype == StreamTokenizer.TT_WORD) {
        if (st.sval.toLowerCase().equals("moore"))
          t.isMooreStyle = true;
        else if (st.sval.toLowerCase().equals("mealy"))
          t.isMooreStyle = false;
        else {
          System.err.println("State table must begin with \"Moore\" or \"Mealy\", "+st.sval);
          System.exit(1);
        }
      }
      else {
        System.err.println("State table must begin with \"Moore\" or \"Mealy\".");
        System.exit(1);
      }

      ttype = st.nextToken();
      if (ttype == StreamTokenizer.TT_NUMBER) {
        int numStates = (int)st.nval;
        t.setSize(numStates);
      }
      else {
        System.err.println("Expecting number of states, not "+st.sval);
        System.exit(1);
      }
    
      ttype = st.nextToken();
      if (ttype == StreamTokenizer.TT_NUMBER) {
        t.numInputs = (int)st.nval;
      }
      else {
        System.err.println("Expecting number of distinct input patterns, not "+st.sval);
        System.exit(1);
      }

      ttype = st.nextToken();
      if (ttype == StreamTokenizer.TT_NUMBER) {
        t.numOutputs = (int)st.nval;
      }
      else {
        System.err.println("Expecting number of distinct output patterns, not "+st.sval);
        System.exit(1);
      }

      for (int s = 0; s < t.size; s++) {
        StateTableRow r = new StateTableRow(t);
        for (int i = 0; i < t.numInputs; i++) {
          ttype = st.nextToken();
          if (ttype != StreamTokenizer.TT_NUMBER) {
            System.err.println("Expecting next state number, not "+st.sval);
            System.exit(1);
          }
          int nextState = (int)st.nval;
          if ((nextState >= t.size) || (nextState < 0)) {
            System.err.println("State "+nextState+ " is out of range (max="+(t.size-1)+")");
            System.exit(1);
          }
          r.nextState[i] = nextState;
        }
        for (int i = 0; i < (t.isMooreStyle ? 1 : t.numInputs); i++) {
          ttype = st.nextToken();
          if (ttype != StreamTokenizer.TT_NUMBER) {
            System.err.println("Expecting output number, not "+st.sval);
            System.exit(1);
          }
          int output = (int)st.nval;
          if ((output >= t.numOutputs) || (output < 0)) {
            System.err.println("Output "+output+ " is out of range (max="+(t.numOutputs-1)+")");
            System.exit(1);
          }
          r.output[i] = output;
        }
        t.rows[s] = r;
      }
    }
    catch (java.io.IOException e) {
      System.err.println("Failed to read the input for some reason");
      System.exit(1);
    }
  }
}
