import java.util.Vector;

// A StateTable is a container for all the information needed for a finite state
// machine
public class StateTable
{
  // If isMooreStyle is true, this is a Moore-style machine.  Otherwise, it is a
  // Mealy style machine.
  public boolean isMooreStyle;
  // size, numInputs and numOutputs define how many states this FSM has, how
  // many different input patterns it recognizes and how many different output
  // patterns it can generate.
  public int size, numInputs, numOutputs;
  // rows is an array of size "size" that stores the next state and output
  // information for this FSM.  The states are implicitly "named" 0..size-1.
  public StateTableRow [] rows;

  // setSize allocates a new array of rows of the appropriate size
  public void setSize(int sz) {
    size = sz;
    rows = new StateTableRow[size];
  }

  public String toString() {
    StringBuffer s = new StringBuffer();
    s.append(isMooreStyle ? "Moore" : "Mealy");
    s.append(" " + size + " " + numInputs + " " + numOutputs + "\n");
    for (int i = 0; i < size; i++) {
      s.append("S" + i + ":  ");
      s.append(rows[i] + "\n");
    }
    return s.toString();
  }
}
