import java.util.Vector;

public class StateTableRow
{
  // Reference to the table that this row is part of
  public StateTable tableRef;
  // Array of next states.  The size of this array is equal to the number of
  // possible input patters.  To find what the next state is for a particular
  // input, simply use the input as an index into this array.  The input
  // patterns that an FSM recognizes are implicitly 0..numInputs-1.
  public int [] nextState;
  // For Mealy machines, the output array is just like the nextState array, only
  // for outputs instead of next states.  For Moore machines, the output array
  // is always size 1.
  public int [] output;

  // Build an empty row
  public StateTableRow(StateTable t) {
    //System.out.println("Making a new row " + t.numInputs + " " + t.numOutputs + " " + t.isMooreStyle);
    tableRef = t;
    nextState = new int[t.numInputs];
    if (t.isMooreStyle)
      output = new int[1];
    else
      output = new int[t.numInputs];
  }

  public String toString() {
    StringBuffer s = new StringBuffer();
    for (int i = 0; i < tableRef.numInputs; i++) {
      s.append(nextState[i] + " ");
    }
    for (int i = 0; i < (tableRef.isMooreStyle ? 1 : tableRef.numInputs); i++) {
      s.append(output[i] + " ");
    }
    return s.toString();
  }
}
